<?php

/**
 * Nette Framework
 *
 * Copyright (c) 2004, 2009 David Grudl (http://davidgrudl.com)
 *
 * This source file is subject to the "Nette license" that is bundled
 * with this package in the file license.txt.
 *
 * For more information please see http://nettephp.com
 *
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @license    http://nettephp.com/license  Nette license
 * @link       http://nettephp.com
 * @category   Nette
 * @package    Loaders
 */



require_once dirname(__FILE__) . '/../Loaders/AutoLoader.php';



/**
 * Nette auto loader is responsible for loading Nette classes and interfaces.
 *
 * @author     David Grudl
 * @copyright  Copyright (c) 2004, 2009 David Grudl
 * @package    Loaders
 */
class NNetteLoader extends NAutoLoader
{
	/** @var NNetteLoader */
	public static $instance;

	/** @var string  base file path */
	public $base;

	/** @var array */
	public $list = array(
		'argumentoutofrangeexception' => '/exceptions.php',
		'deprecatedexception' => '/exceptions.php',
		'directorynotfoundexception' => '/exceptions.php',
		'fatalerrorexception' => '/exceptions.php',
		'filenotfoundexception' => '/exceptions.php',
		'iauthenticator' => '/Security/IAuthenticator.php',
		'iauthorizator' => '/Security/IAuthorizator.php',
		'icachestorage' => '/Caching/ICacheStorage.php',
		'icollection' => '/Collections/ICollection.php',
		'icomponent' => '/IComponent.php',
		'icomponentcontainer' => '/IComponentContainer.php',
		'iconfigadapter' => '/Config/IConfigAdapter.php',
		'idebuggable' => '/IDebuggable.php',
		'ifiletemplate' => '/Templates/IFileTemplate.php',
		'iformcontrol' => '/Forms/IFormControl.php',
		'iformrenderer' => '/Forms/IFormRenderer.php',
		'ihttprequest' => '/Web/IHttpRequest.php',
		'ihttpresponse' => '/Web/IHttpResponse.php',
		'iidentity' => '/Security/IIdentity.php',
		'ilist' => '/Collections/IList.php',
		'imailer' => '/Mail/IMailer.php',
		'imap' => '/Collections/IMap.php',
		'inamingcontainer' => '/Forms/INamingContainer.php',
		'invalidstateexception' => '/exceptions.php',
		'ioexception' => '/exceptions.php',
		'ipartiallyrenderable' => '/Application/IRenderable.php',
		'ipermissionassertion' => '/Security/IPermissionAssertion.php',
		'ipresenter' => '/Application/IPresenter.php',
		'ipresenterloader' => '/Application/IPresenterLoader.php',
		'ipresenterresponse' => '/Application/IPresenterResponse.php',
		'irenderable' => '/Application/IRenderable.php',
		'iresource' => '/Security/IResource.php',
		'irole' => '/Security/IRole.php',
		'irouter' => '/Application/IRouter.php',
		'iservicelocator' => '/IServiceLocator.php',
		'iset' => '/Collections/ISet.php',
		'isignalreceiver' => '/Application/ISignalReceiver.php',
		'istatepersistent' => '/Application/IStatePersistent.php',
		'isubmittercontrol' => '/Forms/ISubmitterControl.php',
		'itemplate' => '/Templates/ITemplate.php',
		'itranslator' => '/ITranslator.php',
		'iuser' => '/Web/IUser.php',
		'lattefilter' => '/Templates/Filters/LatteFilter.php',
		'lattemacros' => '/Templates/Filters/LatteMacros.php',
		'memberaccessexception' => '/exceptions.php',
		'nabortexception' => '/Application/Exceptions/AbortException.php',
		'nambiguousserviceexception' => '/ServiceLocator.php',
		'nannotations' => '/Annotations.php',
		'nappform' => '/Application/AppForm.php',
		'napplication' => '/Application/Application.php',
		'napplicationexception' => '/Application/Exceptions/ApplicationException.php',
		'narraylist' => '/Collections/ArrayList.php',
		'narraytools' => '/ArrayTools.php',
		'nauthenticationexception' => '/Security/AuthenticationException.php',
		'nautoloader' => '/Loaders/AutoLoader.php',
		'nbadrequestexception' => '/Application/Exceptions/BadRequestException.php',
		'nbadsignalexception' => '/Application/Exceptions/BadSignalException.php',
		'nbasetemplate' => '/Templates/BaseTemplate.php',
		'nbutton' => '/Forms/Controls/Button.php',
		'ncache' => '/Caching/Cache.php',
		'ncachinghelper' => '/Templates/Filters/CachingHelper.php',
		'ncheckbox' => '/Forms/Controls/Checkbox.php',
		'nclirouter' => '/Application/Routers/CliRouter.php',
		'ncollection' => '/Collections/Collection.php',
		'ncomponent' => '/Component.php',
		'ncomponentcontainer' => '/ComponentContainer.php',
		'nconfig' => '/Config/Config.php',
		'nconfigadapterini' => '/Config/ConfigAdapterIni.php',
		'nconfigadapterxml' => '/Config/ConfigAdapterXml.php',
		'nconfigurator' => '/Configurator.php',
		'ncontrol' => '/Application/Control.php',
		'nconventionalrenderer' => '/Forms/Renderers/ConventionalRenderer.php',
		'ncurlybracketsfilter' => '/Templates/Filters/LatteFilter.php',
		'ncurlybracketsmacros' => '/Templates/Filters/LatteFilter.php',
		'ndebug' => '/Debug.php',
		'ndownloadresponse' => '/Application/Responses/DownloadResponse.php',
		'ndummystorage' => '/Caching/DummyStorage.php',
		'nenvironment' => '/Environment.php',
		'nfilestorage' => '/Caching/FileStorage.php',
		'nfileupload' => '/Forms/Controls/FileUpload.php',
		'nform' => '/Forms/Form.php',
		'nformcontainer' => '/Forms/FormContainer.php',
		'nformcontrol' => '/Forms/Controls/FormControl.php',
		'nformgroup' => '/Forms/FormGroup.php',
		'nforwardingresponse' => '/Application/Responses/ForwardingResponse.php',
		'nframework' => '/Framework.php',
		'nfreezableobject' => '/FreezableObject.php',
		'nftp' => '/Web/Ftp.php',
		'nftpexception' => '/Web/Ftp.php',
		'nhashtable' => '/Collections/Hashtable.php',
		'nhiddenfield' => '/Forms/Controls/HiddenField.php',
		'nhtml' => '/Web/Html.php',
		'nhttprequest' => '/Web/HttpRequest.php',
		'nhttpresponse' => '/Web/HttpResponse.php',
		'nhttpuploadedfile' => '/Web/HttpUploadedFile.php',
		'nidentity' => '/Security/Identity.php',
		'nimage' => '/Image.php',
		'nimagebutton' => '/Forms/Controls/ImageButton.php',
		'nimagemagick' => '/ImageMagick.php',
		'ninstancefilteriterator' => '/InstanceFilterIterator.php',
		'ninstantclientscript' => '/Forms/Renderers/InstantClientScript.php',
		'ninvalidlinkexception' => '/Application/Exceptions/InvalidLinkException.php',
		'ninvalidpresenterexception' => '/Application/Exceptions/InvalidPresenterException.php',
		'njsonresponse' => '/Application/Responses/JsonResponse.php',
		'nkeynotfoundexception' => '/Collections/Hashtable.php',
		'nlimitedscope' => '/Loaders/LimitedScope.php',
		'nlink' => '/Application/Link.php',
		'nmail' => '/Mail/Mail.php',
		'nmailmimepart' => '/Mail/MailMimePart.php',
		'nmemcachedstorage' => '/Caching/MemcachedStorage.php',
		'nmultirouter' => '/Application/Routers/MultiRouter.php',
		'nmultiselectbox' => '/Forms/Controls/MultiSelectBox.php',
		'nnetteloader' => '/Loaders/NetteLoader.php',
		'nobject' => '/Object.php',
		'nobjectmixin' => '/ObjectMixin.php',
		'notimplementedexception' => '/exceptions.php',
		'notsupportedexception' => '/exceptions.php',
		'npaginator' => '/Paginator.php',
		'npermission' => '/Security/Permission.php',
		'npresenter' => '/Application/Presenter.php',
		'npresentercomponent' => '/Application/PresenterComponent.php',
		'npresenterhelpers' => '/Application/PresenterHelpers.php',
		'npresenterloader' => '/Application/PresenterLoader.php',
		'npresenterrequest' => '/Application/PresenterRequest.php',
		'nradiolist' => '/Forms/Controls/RadioList.php',
		'nrecursivecomponentiterator' => '/ComponentContainer.php',
		'nrecursivehtmliterator' => '/Web/Html.php',
		'nredirectingresponse' => '/Application/Responses/RedirectingResponse.php',
		'nrenderresponse' => '/Application/Responses/RenderResponse.php',
		'nrepeatercontrol' => '/Forms/Controls/RepeaterControl.php',
		'nrobotloader' => '/Loaders/RobotLoader.php',
		'nroute' => '/Application/Routers/Route.php',
		'nrule' => '/Forms/Rule.php',
		'nrules' => '/Forms/Rules.php',
		'nsafestream' => '/IO/SafeStream.php',
		'nselectbox' => '/Forms/Controls/SelectBox.php',
		'nsendmailmailer' => '/Mail/SendmailMailer.php',
		'nservicelocator' => '/ServiceLocator.php',
		'nsession' => '/Web/Session.php',
		'nsessionnamespace' => '/Web/SessionNamespace.php',
		'nset' => '/Collections/Set.php',
		'nsimpleauthenticator' => '/Security/SimpleAuthenticator.php',
		'nsimpleloader' => '/Loaders/SimpleLoader.php',
		'nsimplerouter' => '/Application/Routers/SimpleRouter.php',
		'nsmartcachingiterator' => '/SmartCachingIterator.php',
		'nsnippethelper' => '/Templates/Filters/SnippetHelper.php',
		'nstring' => '/String.php',
		'nsubmitbutton' => '/Forms/Controls/SubmitButton.php',
		'ntemplate' => '/Templates/Template.php',
		'ntemplatecachestorage' => '/Templates/TemplateCacheStorage.php',
		'ntemplatefilters' => '/Templates/Filters/TemplateFilters.php',
		'ntemplatehelpers' => '/Templates/Filters/TemplateHelpers.php',
		'ntextarea' => '/Forms/Controls/TextArea.php',
		'ntextbase' => '/Forms/Controls/TextBase.php',
		'ntextinput' => '/Forms/Controls/TextInput.php',
		'ntools' => '/Tools.php',
		'nuri' => '/Web/Uri.php',
		'nuriscript' => '/Web/UriScript.php',
		'nuser' => '/Web/User.php',
		'nuserclientscript' => '/Forms/Renderers/UserClientScript.php',
	);



	/**
	 * Returns singleton instance with lazy instantiation.
	 * @return NNetteLoader
	 */
	public static function getInstance()
	{
		if (self::$instance === NULL) {
			self::$instance = new self;
		}
		return self::$instance;
	}



	/**
	 * Handles autoloading of classes or interfaces.
	 * @param  string
	 * @return void
	 */
	public function tryLoad($type)
	{
		$type = strtolower($type);
		if (isset($this->list[$type])) {
			NLimitedScope::load($this->base . $this->list[$type]);
			self::$count++;
		}
	}

}
